@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -
 #
 # This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
@@ -24,6 +23,7 @@ from gevent import hub, monkey, socket, pywsgi
 
 import gunicorn
 from gunicorn.http.wsgi import base_environ
+from gunicorn.sock import ssl_context
 from gunicorn.workers.base_async import AsyncWorker
 
 VERSION = "gevent/%s gunicorn/%s" % (gevent.__version__, gunicorn.__version__)
@@ -41,7 +41,7 @@ class GeventWorker(AsyncWorker):
         sockets = []
         for s in self.sockets:
             sockets.append(socket.socket(s.FAMILY, socket.SOCK_STREAM,
-                fileno=s.sock.fileno()))
+                                         fileno=s.sock.fileno()))
         self.sockets = sockets
 
     def notify(self):
@@ -58,7 +58,7 @@ class GeventWorker(AsyncWorker):
         ssl_args = {}
 
         if self.cfg.is_ssl:
-            ssl_args = dict(server_side=True, **self.cfg.ssl_options)
+            ssl_args = {"ssl_context": ssl_context(self.cfg)}
 
         for s in self.sockets:
             s.setblocking(1)
@@ -110,7 +110,7 @@ class GeventWorker(AsyncWorker):
                 gevent.sleep(1.0)
 
             # Force kill all active the handlers
-            self.log.warning("Worker graceful timeout (pid:%s)" % self.pid)
+            self.log.warning("Worker graceful timeout (pid:%s)", self.pid)
             for server in servers:
                 server.stop(timeout=1)
         except Exception:
@@ -146,7 +146,7 @@ class GeventWorker(AsyncWorker):
         super().init_process()
 
 
-class GeventResponse(object):
+class GeventResponse:
 
     status = None
     headers = None
@@ -165,7 +165,11 @@ class PyWSGIHandler(pywsgi.WSGIHandler):
         finish = datetime.fromtimestamp(self.time_finish)
         response_time = finish - start
         resp_headers = getattr(self, 'response_headers', {})
-        resp = GeventResponse(self.status, resp_headers, self.response_length)
+
+        # Status is expected to be a string but is encoded to bytes in gevent for PY3
+        # Except when it isn't because gevent uses hardcoded strings for network errors.
+        status = self.status.decode() if isinstance(self.status, bytes) else self.status
+        resp = GeventResponse(status, resp_headers, self.response_length)
         if hasattr(self, 'headers'):
             req_headers = self.headers.items()
         else:
